;Allows you to show a message on the overworld
;Actual data should go into "_ow_messages_data.asm"

incsrc "_ow_messages_settings.asm"

!BlankTileProperties = $39
!LockedMessageProperties = $39
!UnlockedMessageProperties = $39
!TextProperties = $3D

!LvlNumMax #= $13B
!AutoTranslateLvlNum #= 1 ;Set to zero if you want to adjust level numbers
						  ;between $101-$13B manually for some reason.
						  ;If you do that, then don't forget to adjust
						  ;!LvlNumMax up there as well.
						  
;Do not adjust the defines below.
!OwMsgAmnt #= 0
						  
!MsgState = $147A|!addr		;state ,1 byte 
!WinVPos = $147C|!addr		;vertical window pos, 1 byte
!IndexBackup = $147E|!addr	;backup of the message data index, 2 bytes
!StripeHeader = $1480|!addr	;stripe string header to use on the next line, 2 bytes
!LinesLeft = $1483|!addr	;number of lines left to draw, 1 byte
!WinHPosL = $1484|!addr		;left window pos, 1 byte
!WinHPosR = $1485|!addr		;right window pos, 1 byte

!True = 1
!False = 0

macro AddOWMessage(msglabel,lvlnum,spoiler)
	;<spoiler> should be a boolean, passed to this macro using !True or !False (1 or 0)
	assert <spoiler> == !True || <spoiler> == !False, "Third parameter to AddOWMessage macro is invalid."
	!AutoTranslateLvlNum ?= 0
	
	!lvlnum #= <lvlnum>
	if !AutoTranslateLvlNum != 0
		assert !lvlnum <= $24 || !lvlnum >= $101
		assert !lvlnum <= $13B
				
		if !lvlnum >= $101
			!lvlnum #= !lvlnum-$DC
		endif
	endif

	;This basically makes an imaginary array of tuples.
	;The tuples' three members are accessed through this syntax:
	;!OwMsgsLbl[index]
	;!OwMsgsNum[index]
	;!OwMsgsSpoiler[index]
	;Replace [index] with the index number (example: "!OwMsgsNum5")
	;The array is zero-indexed.
	
	;Each time this macro is called it pushes an entry onto it.
	;!OwMsgAmnt represents this array's length, which is used
	;for loops that iterate through it.
	
	!{OwMsgsLbl!{OwMsgAmnt}} = <msglabel>
	!{OwMsgsNum!{OwMsgAmnt}} #= !lvlnum
	!{OwMsgsSpoiler!{OwMsgAmnt}} #= <spoiler>
	
	!OwMsgAmnt #= !OwMsgAmnt+1
endmacro

macro CheckIfLvlHasMsg(iterator)	;'Return value' is in !HasMsg. '!True' for true, '!False' for false.
	!HasMsg = !False				;Also sets !LvlIndex to the level's index in our imaginary array,
	!CheckIter #= 0					;but only if it actually exists.
	while !CheckIter != !OwMsgAmnt
		if !{OwMsgsNum!{CheckIter}} == <iterator>
			!HasMsg = !True
			!LvlIndex #= !CheckIter
		endif
		!CheckIter #= !CheckIter+1
	endif
	
endmacro

macro FinalizeOWMessages()
	;First adjust max level number.
	if !AutoTranslateLvlNum != 0
		assert !LvlNumMax <= $24 || !LvlNumMax >= $101
		assert !LvlNumMax <= $13B
		
		!LvlNumMax #= !LvlNumMax-$DC
	endif
	
	assert (!LvlNumMax+1)%4 == 0,"Error: Level amount needs to be a multiple of four."

	;Next populate the settings table.
	Settings:
	!SettingsLoopIter #= 0
	while !SettingsLoopIter <= !LvlNumMax
		;Go into nested loop to create a single settings byte (settings for four levels)
		!NextByteLvl #= !SettingsLoopIter+4
		!BytePos #= 0
		while !SettingsLoopIter != !NextByteLvl
			!{SetBytePart!{BytePos}} #= 0
			%CheckIfLvlHasMsg(!SettingsLoopIter)
			if !HasMsg == !True
				;Set the bit that indicates there's a message.
				!{SetBytePart!{BytePos}} #= %00000001
				if !{OwMsgsSpoiler!{LvlIndex}} == 1
					;If it's a spoiler, set the second bit as well.
					!{SetBytePart!{BytePos}} #= (!{SetBytePart!{BytePos}}|%00000010)
				endif
			endif
			!SettingsLoopIter #= !SettingsLoopIter+1
			!BytePos #= !BytePos+1
		endif
		;Now combine the results into one settings byte, and write it.
		db ((!SetBytePart0<<6)|(!SetBytePart1<<4)|(!SetBytePart2<<2)|(!SetBytePart3))
		
	endif
	
	;Finally, populate the message pointer table.
	Pointers:
	!PointersLoopIter #= 0
	while !PointersLoopIter <= !LvlNumMax
		%CheckIfLvlHasMsg(!PointersLoopIter)
		if !HasMsg == !True
			dw !{OwMsgsLbl!{LvlIndex}}-Message
		else
			dw $0000
		endif
		!PointersLoopIter #= !PointersLoopIter+1
	endif
endmacro

!addr = $0000
if read1($00FFD5) == $23
	sa1rom
	!addr = $6000
endif

assert read1($048E81) == $22, "Level Name hijack not installed"


org $048E89	;replaces 2 NOPs used by the level name hijack (some patches may replace the 4 NOPs before)
autoclean JSL PreIconic1

org $049551	;same as above
autoclean JSL PreIconic2
NOP

org $048281	;main code
autoclean JML MainCode
NOP


freedata

PreIconic1:
JSR Iconic
LDX $04
RTL
PreIconic2:
JSR Iconic
INC $13D9|!addr
RTL

Iconic:
PHX
PHY
LDA $00
PHA
LDA $02
PHA
SEP #$30
LDA $16
PHA
STZ $16
JSR Wait
PLA
STA $16
REP #$30
PLA
STA $02
PLA
STA $00
PLY
PLX
RTS


MainCode:
LDA $1B87|!addr
BEQ +
JML $048286
+
LDA $13D4|!addr
BNE +
LDA $13D9
CMP #$03
BNE +
LDA !MsgState
ASL A
TAX
JSR (State,x)
SEP #$30
LDA !MsgState
BEQ +
REP #$20
STZ $15
STZ $17
STZ $0DA2
STZ $0DA4
STZ $0DA6
STZ $0DA8
SEP #$20
+
JML $048295


State:
dw Wait,WindowInit,WindowH,WindowV,Idle,CloseV,CloseH,CloseEnd

Wait:
	LDX $0DD6|!addr
	LDY $0DB3|!addr
	REP #$30
	LDA.w $1F1F|!addr,x
	AND #$000F
	STA.b $00
	LDA.w $1F21|!addr,x
	AND #$000F
	ASL #4
	STA.b $02
	LDA.w $1F1F|!addr,x
	AND #$0010
	ASL #4
	ORA.b $00
	STA.b $00
	LDA.w $1F21|!addr,x
	AND #$0010
	ASL #5
	ORA.b $02
	ORA.b $00
	TAX
	LDA.w $1F11|!addr,y
	AND #$000F
	BEQ +
	TXA
	CLC
	ADC #$0400
	TAX
+
LDA.l $7ED000,x
SEP #$30
STA $13BF|!addr
LSR #2
TAX
LDA $13BF|!addr
AND #$03
TAY
LDA.l Settings,x	;Start checking note settings
DEY
BMI +
-
ASL #2			;Code to shift the level's two settings bits into the right place in the byte
DEY
BPL -
+
AND #$C0
BEQ ++			;If neither bit is set, jump ahead (no message)
CMP #$C0
BNE +			;If only one bit is set, jump ahead (message, unlocked)
LDX $13BF|!addr
LDA $1EA2|!addr,x
AND #$80
BNE +			;If 'level beaten' flag for the level is set, jump ahead (message, unlocked)
LDA #!LockedMessageIcon
LDX #!LockedMessageProperties
BRA +++			;Then jump to JMP to DrawIcon
+
JSR Idle
STZ $41
STZ $43
LDA #!UnlockedMessageIcon
LDX #!UnlockedMessageProperties
BRA +++
++
LDA #!BlankTile
LDX #!BlankTileProperties
+++
JMP DrawIcon
RTS

Idle:
LDA $16
AND #$20
BEQ +
INC !MsgState
+
RTS

DrawIcon:
PHB
LDY #$7F
PHY
PLB

PHX
PHA

REP #$30
LDA $837B
TAY
CLC
ADC #$0006
STA $837B
LDA #$9F50
STA $837D,y
LDA #$0100
STA $837F,y
PLA
STA $8381,y
SEP #$20
LDA #$FF
STA $8383,y
SEP #$10
PLB
RTS


CloseV:
REP #$30

LDA !WinVPos
AND #$00FF
ASL A
TAX
LDA #$00FF
STA $04FC|!addr,x
TXA
LSR A
SEP #$30
DEC !WinVPos
DEC
BEQ +
AND #$07
BNE +

JSR MsgClear
+
LDA !WinVPos
BNE +
INC !MsgState
+
RTS


CloseH:
LDA !WinHPosL
INC #2
STA !WinHPosL
STA $04FC|!addr

LDA !WinHPosR
DEC #2
STA !WinHPosR
STA $04FD|!addr
CMP #$80
BNE +
INC !MsgState
STZ $41
STZ $43
+
RTS


CloseEnd:
LDA #$1C		;the sound that plays when the message has been closed
STA $1DFC|!addr
STZ !MsgState
REP #$30
LDX #$01BE
LDA #$FF00
-
STA $04A0|!addr,x
DEX #2
BPL -
SEP #$30
STZ !LinesLeft
RTS


WindowInit:
LDA #$22		;the sound that plays when the message starts being drawn
STA $1DFC|!addr
LDA #$80
STA $0D9F|!addr
LDA #$7F
STA !WinHPosL
STA $04FC|!addr
INC
STA !WinHPosR
STA $04FD|!addr
REP #$30
LDA #$0000
STA $0701|!addr
LDX #$01BE
LDA #$00FF
-
STA $04A0|!addr,x
DEX #2
BPL -
SEP #$30
STZ !WinVPos
INC !MsgState
RTS


WindowH:
LDA #$22
STA $41
LDA #$22
STA $43
LDA !WinHPosL
DEC #2
STA !WinHPosL
STA $04FC|!addr
LDA !WinHPosR
INC #2
STA !WinHPosR
STA $04FD|!addr
CMP #$E8
BNE +
INC !MsgState
+
RTS


WindowV:
INC !WinVPos
REP #$30
LDA !WinVPos
AND #$00FF
ASL A
TAX
LDA !WinHPosL
STA $04FC|!addr,x
TXA
LSR A
SEP #$30
DEC
BEQ +
AND #$07
BNE +

JSR MsgDraw
DEC !LinesLeft
BNE +
INC !MsgState
+
RTS


MsgClear:
REP #$30
LDA !StripeHeader
XBA
SEC
SBC #$0020
XBA
STA !StripeHeader

PHB
PEA $7F7F
PLB
PLB

LDA $837B
TAY
CLC
ADC #$0006
STA $837B
LDA.l !StripeHeader
STA $837D
LDA #$3240
STA $837F,y
LDA #$00FC
STA $8381,y
SEP #$20
LDA #$FF
STA $8383,y
REP #$30
PLB
RTS

MsgDraw:
LDA !LinesLeft
REP #$30
BEQ +

LDA !StripeHeader
STA $02
LDX !IndexBackup
BRA skip

+
LDA $13BF|!addr
AND #$00FF
ASL A
TAX

LDA.l Pointers,x
TAX

LDA.l Message,x
INX
SEP #$20
STA !LinesLeft
REP #$20

LDA #$C350
STA $02

skip:
PHB
PEA $7F7F
PLB
PLB

LDA $837B
TAY

CLC
ADC #$0038
STA $837B

LDA $02
STA $837D,y
LDA #$3300
STA $837F,y
INY #4
SEP #$20
-
LDA.l Message,x
 CMP #$FF
 BNE +
 LDA #$1F
 DEX
+ STA $837D,y
 LDA #!TextProperties
 STA $837E,y
 INY #2
 INX
CPY $837B
BCC -

LDA #$FF
STA $837D,y
PLB
INX
STX !IndexBackup
REP #$20
LDA $02
XBA
CLC
ADC #$0020
XBA
STA !StripeHeader
SEP #$20
RTS

incsrc _ow_messages_data.asm